#!/bin/bash

# ./spaces.sh [file-path] [-f | --fix] [-h | --help]

usage () {
   echo "Usage: $0 [file-path] [-f | --fix] [-h | --help]"
   exit 1
}

FIX=0     #0 - do not perform fix       1 - perform fix

if [ $# -eq 0 ]; then
   usage
fi


while [ $# -gt 0 ]
do
   case "$1" in
      -f|--fix )
         FIX=1
         shift
         ;;
      -h|--help )
         usage
         shift
         ;;
      * )
         if [ -f "$1" ]; then
            FILE="$1"
            shift
         else
            usage
         fi
         ;;

   esac
done


if [ $FIX -eq 1 ] && [ -f "$FILE" ]; then
   echo "Fixing spaces and tabs at the beginning and at the end of lines"
   sed -i 's/[[:blank:]]\+$//' "$FILE"
   sed -i 's/^[[:blank:]]\+//' "$FILE"
fi


# display graphically space errors
if [ -f "$FILE" ]; then
   LINENUMBER=0
   REGEX_START="^[[:blank:]]+"  # bash uses extended regex, thus we don't need the backslash before the + symbol
   REGEX_END="[[:blank:]]+$"

   while IFS= read -r line
   do
      let LINENUMBER++ #incrementing the variable LINES
      #if there is no space issue on a line, just print the line then
      echo "$line" | sed -e '/[[:blank:]]\+$/q9' -e '/^[[:blank:]]\+/q7' >/dev/null
      if [ $? -eq 0 ]; then
         printf %4s "$LINENUMBER: " >> temp.txt
         #by printing printf %4s we are saying that we want to have the value of the variable LINES printed in the place of the 4 first characters
         echo "$line" >> temp.txt
         continue #it finishes the actual loop and proceeds with the next one, in this case it will jump to the next line in the while loop (but we could use the else statement as well)
      fi

      # print line number
      printf %4s "$LINENUMBER: " >> temp.txt

      # print the spaces and tabs at the beginning of the requested line on the red background
      if [[ "$line" =~ $REGEX_START ]]; then
         MATCH=$(echo "$BASH_REMATCH" | sed 's/\t/|___TAB___|/g')
         echo -e -n "\e[41m$BASH_REMATCH\e[49m" >> temp.txt 
      fi

      # print the lines that don't contain spaces and tabs at their beginning and the end
      echo -e -n "$line" | sed -e 's/^[[:blank:]]\+//' -e 's/[[:blank:]]\+$//' >> temp.txt

      # print the lines that contain spaces and tabs at their end on the red background
      if [[ "$line" =~ $REGEX_END ]]; then
         MATCH=$(echo "$BASH_REMATCH" | sed 's/\t/|___TAB___|/g')
         echo -e "\e[41m$BASH_REMATCH\e[49m" >> temp.txt 
      else
         echo >> temp.txt
      fi

   done < "$FILE"

   cat temp.txt
   rm temp.txt

fi

if [ $FIX -eq 1 ]; then
   echo 
   echo -e "\e[42mDONE\e[49m"
fi
